@@ -45,12 +45,15 @@ long cm_AddCellProc(void *rockp, struct sockaddr_in *addrp, char *namep)
 			tsp->cellp = cellp;
 	}
 	else
-        	tsp = cm_NewServer(addrp, CM_SERVER_VLDB, cellp);
+        tsp = cm_NewServer(addrp, CM_SERVER_VLDB, cellp);
 
 	/* Insert the vlserver into a sorted list, sorted by server rank */
 	tsrp = cm_NewServerRef(tsp);
 	cm_InsertServerList(&cellp->vlServersp, tsrp);
-
+    /* drop the allocation reference */
+    lock_ObtainWrite(&cm_serverLock);
+    tsrp->refCount--;
+    lock_ReleaseWrite(&cm_serverLock);
 	return 0;
 }
 
@@ -92,22 +95,37 @@ cm_cell_t *cm_GetCell_Gen(char *namep, char *newnamep, long flags)
             memset(cp, 0, sizeof(cm_cell_t));
         } 
         else {
+            cm_cell_t **cpp;
+
             dns_expired = 1;
-            /* must empty cp->vlServersp */
-            cm_FreeServerList(&cp->vlServersp);
-            cp->vlServersp = NULL;
+            if (cp->vlServersp) {
+                /* must empty cp->vlServersp */
+                lock_ObtainMutex(&cp->mx);
+                cm_FreeServerList(&cp->vlServersp);
+                cp->vlServersp = NULL;
+                lock_ReleaseMutex(&cp->mx);
+            }
+            /* remove the entry from the allCells list 
+             * we will re-insert it later 
+             */
+            for (cpp = &cm_allCellsp; *cpp; cpp=&(*cpp)->nextp) {
+                if (*cpp == cp) {
+                    (*cpp) = cp->nextp;
+                    break;
+                }
+            }
         }
 
         code = cm_SearchCellFile(namep, fullname, cm_AddCellProc, cp);
 		if (code) {
-            afsi_log("in cm_GetCell_gen cm_SearchCellFile(%s) returns code= %d fullname= %s", 
+            osi_Log3(afsd_logp,"in cm_GetCell_gen cm_SearchCellFile(%s) returns code= %d fullname= %s", 
                       namep, code, fullname);
 
 #ifdef AFS_AFSDB_ENV
             if (cm_dnsEnabled /*&& cm_DomainValid(namep)*/) {
                 code = cm_SearchCellByDNS(namep, fullname, &ttl, cm_AddCellProc, cp);
                 if ( code ) {
-                    afsi_log("in cm_GetCell_gen cm_SearchCellByDNS(%s) returns code= %d fullname= %s", 
+                    osi_Log3(afsd_logp,"in cm_GetCell_gen cm_SearchCellByDNS(%s) returns code= %d fullname= %s", 
                              namep, code, fullname);
                     if (dns_expired) {
                         cp->flags |= CM_CELLFLAG_VLSERVER_INVALID;
