@@ -14,7 +14,7 @@
  * afs_IsDynrootFid
  * afs_GetDynrootFid
  * afs_IsDynroot
- * afs_RefreshDynroot
+ * afs_DynrootInvalidate
  * afs_GetDynroot
  * afs_PutDynroot
  * afs_DynrootNewVnode
@@ -26,19 +26,19 @@
  */
 
 #include <afsconfig.h>
-#include "../afs/param.h"
+#include "afs/param.h"
 
-#include "../afs/stds.h"
-#include "../afs/sysincludes.h" /* Standard vendor system headers */
-#include "../afs/afsincludes.h"
-#include "../afs/afs_osi.h"
-#include "../afsint/afsint.h"
-#include "../afs/lock.h"
+#include "afs/stds.h"
+#include "afs/sysincludes.h"	/* Standard vendor system headers */
+#include "afsincludes.h"
+#include "afs/afs_osi.h"
+#include "afsint.h"
+#include "afs/lock.h"
 
-#include "../afs/prs_fs.h"
-#include "../afs/dir.h"
+#include "afs/prs_fs.h"
+#include "afs/dir.h"
 
-#define AFS_DYNROOT_CELL	1
+#define AFS_DYNROOT_CELLNAME	"dynroot"
 #define AFS_DYNROOT_VOLUME	1
 #define AFS_DYNROOT_VNODE	1
 #define AFS_DYNROOT_UNIQUE	1
@@ -48,7 +48,8 @@
  * and a type-specific identifier in the lower 24 bits.
  */
 #define VN_TYPE_CELL		0x01	/* Corresponds to a struct cell */
-#define VN_TYPE_SYMLINK		0x02	/* User-created symlink in /afs */
+#define VN_TYPE_ALIAS		0x02	/* Corresponds to a struct cell_alias */
+#define VN_TYPE_SYMLINK		0x03	/* User-created symlink in /afs */
 
 #define VNUM_TO_VNTYPE(vnum)	((vnum) >> 24)
 #define VNUM_TO_VNID(vnum)	((vnum) & 0x00ffffff)
@@ -59,15 +60,19 @@
 #define VNUM_FROM_CIDX_RW(cidx, rw) \
 				VNUM_FROM_TYPEID(VN_TYPE_CELL, \
 						 ((cidx) << 2 | (rw) << 1))
+#define VNUM_FROM_CAIDX_RW(caidx, rw) \
+				VNUM_FROM_TYPEID(VN_TYPE_ALIAS, \
+						 ((caidx) << 2 | (rw) << 1))
 
 static int afs_dynrootEnable = 0;
+static int afs_dynrootCell = 0;
 
 static afs_rwlock_t afs_dynrootDirLock;
 /* Start of variables protected by afs_dynrootDirLock */
 static char *afs_dynrootDir = NULL;
 static int afs_dynrootDirLen;
 static int afs_dynrootDirLinkcnt;
-static int afs_dynrootCellCount;
+static int afs_dynrootDirVersion;
 static int afs_dynrootVersion = 1;
 static int afs_dynrootVersionHigh = 1;
 /* End of variables protected by afs_dynrootDirLock */
@@ -86,8 +91,32 @@ static struct afs_dynSymlink *afs_dynSymlinkBase = NULL;
 static int afs_dynSymlinkIndex = 0;
 /* End of variables protected by afs_dynSymlinkLock */
 
-extern afs_int32 afs_cellindex;
-extern afs_rwlock_t afs_xvcache;
+/*
+ * Set up a cell for dynroot if it's not there yet.
+ */
+static int
+afs_dynrootCellInit()
+{
+    if (afs_dynrootEnable && !afs_dynrootCell) {
+	afs_int32 cellHosts[MAXCELLHOSTS];
+	struct cell *tc;
+	int code;
+
+	memset(cellHosts, 0, sizeof(cellHosts));
+	code =
+	    afs_NewCell(AFS_DYNROOT_CELLNAME, cellHosts, CNoSUID | CNoAFSDB,
+			NULL, 0, 0, 0);
+	if (code)
+	    return code;
+	tc = afs_GetCellByName(AFS_DYNROOT_CELLNAME, READ_LOCK);
+	if (!tc)
+	    return ENODEV;
+	afs_dynrootCell = tc->cellNum;
+	afs_PutCell(tc, READ_LOCK);
+    }
+
+    return 0;
+}
 
 /*
  * Returns non-zero iff fid corresponds to the top of the dynroot volume.
@@ -95,23 +124,21 @@ extern afs_rwlock_t afs_xvcache;
 int
 afs_IsDynrootFid(struct VenusFid *fid)
 {
-    return
-	(afs_dynrootEnable &&
-	 fid->Cell       == AFS_DYNROOT_CELL   &&
-	 fid->Fid.Volume == AFS_DYNROOT_VOLUME &&
-	 fid->Fid.Vnode  == AFS_DYNROOT_VNODE  &&
-	 fid->Fid.Unique == AFS_DYNROOT_UNIQUE);
+    return (afs_dynrootEnable && fid->Cell == afs_dynrootCell
+	    && fid->Fid.Volume == AFS_DYNROOT_VOLUME
+	    && fid->Fid.Vnode == AFS_DYNROOT_VNODE
+	    && fid->Fid.Unique == AFS_DYNROOT_UNIQUE);
 }
 
 /*
  * Obtain the magic dynroot volume Fid.
  */
 void
-afs_GetDynrootFid(struct VenusFid *fid) 
+afs_GetDynrootFid(struct VenusFid *fid)
 {
-    fid->Cell       = AFS_DYNROOT_CELL;
+    fid->Cell = afs_dynrootCell;
     fid->Fid.Volume = AFS_DYNROOT_VOLUME;
-    fid->Fid.Vnode  = AFS_DYNROOT_VNODE;
+    fid->Fid.Vnode = AFS_DYNROOT_VNODE;
     fid->Fid.Unique = AFS_DYNROOT_UNIQUE;
 }
 
@@ -119,26 +146,39 @@ afs_GetDynrootFid(struct VenusFid *fid)
  * Returns non-zero iff avc is a pointer to the dynroot /afs vnode.
  */
 int
-afs_IsDynroot(avc)
-    struct vcache *avc;
+afs_IsDynroot(struct vcache *avc)
 {
     return afs_IsDynrootFid(&avc->fid);
 }
 
+/*
+ * Given the current page and chunk pointers in a directory, adjust them
+ * appropriately so that the given file name can be appended.  Used for
+ * computing the size of a directory.
+ */
+static void
+afs_dynroot_computeDirEnt(char *name, int *curPageP, int *curChunkP)
+{
+    int esize;
+
+    esize = afs_dir_NameBlobs(name);
+    if (*curChunkP + esize > EPP) {
+	*curPageP += 1;
+	*curChunkP = 1;
+    }
+    *curChunkP += esize;
+}
+
 /*
  * Add directory entry by given name to a directory.  Assumes the
  * caller has allocated the directory to be large enough to hold
  * the necessary entry.
  */
 static void
-afs_dynroot_addDirEnt(dirHeader, curPageP, curChunkP, name, vnode)
-    struct DirHeader *dirHeader;
-    int *curPageP;
-    int *curChunkP;
-    char *name;
-    int vnode;
+afs_dynroot_addDirEnt(struct DirHeader *dirHeader, int *curPageP,
+		      int *curChunkP, char *name, int vnode)
 {
-    char *dirBase = (char *) dirHeader;
+    char *dirBase = (char *)dirHeader;
     struct PageHeader *pageHeader;
     struct DirEntry *dirEntry;
     int sizeOfEntry, i, t1, t2;
@@ -156,23 +196,23 @@ afs_dynroot_addDirEnt(dirHeader, curPageP, curChunkP, name, vnode)
 	didNewPage = 1;
     }
 
-    pageHeader = (struct PageHeader *) (dirBase + curPage * AFS_PAGESIZE);
+    pageHeader = (struct PageHeader *)(dirBase + curPage * AFS_PAGESIZE);
     if (didNewPage) {
 	pageHeader->pgcount = 0;
 	pageHeader->tag = htons(1234);
 	pageHeader->freecount = 0;
 	pageHeader->freebitmap[0] = 0x01;
-	for (i = 1; i < EPP/8; i++)
+	for (i = 1; i < EPP / 8; i++)
 	    pageHeader->freebitmap[i] = 0;
 
 	dirHeader->alloMap[curPage] = EPP - 1;
     }
 
-    dirEntry = (struct DirEntry *) (pageHeader + curChunk);
-    dirEntry->flag        = 1;
-    dirEntry->length      = 0;
-    dirEntry->next        = 0;
-    dirEntry->fid.vnode   = htonl(vnode);
+    dirEntry = (struct DirEntry *)(pageHeader + curChunk);
+    dirEntry->flag = 1;
+    dirEntry->length = 0;
+    dirEntry->next = 0;
+    dirEntry->fid.vnode = htonl(vnode);
     dirEntry->fid.vunique = htonl(1);
     strcpy(dirEntry->name, name);
 
@@ -196,32 +236,62 @@ afs_dynroot_addDirEnt(dirHeader, curPageP, curChunkP, name, vnode)
     *curChunkP = curChunk;
 }
 
+/*
+ * Invalidate the /afs vnode for dynroot; called when the underlying
+ * directory has changed and needs to be re-read.
+ */
+void
+afs_DynrootInvalidate(void)
+{
+    afs_int32 retry;
+    struct vcache *tvc;
+    struct VenusFid tfid;
+
+    if (!afs_dynrootEnable)
+	return;
+
+    ObtainWriteLock(&afs_dynrootDirLock, 687);
+    afs_dynrootVersion++;
+    afs_dynrootVersionHigh = osi_Time();
+    ReleaseWriteLock(&afs_dynrootDirLock);
+
+    afs_GetDynrootFid(&tfid);
+    do {
+	retry = 0;
+	ObtainReadLock(&afs_xvcache);
+	tvc = afs_FindVCache(&tfid, &retry, 0);
+	ReleaseReadLock(&afs_xvcache);
+    } while (retry);
+    if (tvc) {
+	tvc->states &= ~(CStatd | CUnique);
+	osi_dnlc_purgedp(tvc);
+	afs_PutVCache(tvc);
+    }
+}
+
 /*
  * Regenerates the dynroot contents from the current list of
  * cells.  Useful when the list of cells has changed due to
  * an AFSDB lookup, for instance.
  */
-void
-afs_RefreshDynroot()
+static void
+afs_RebuildDynroot(void)
 {
     int cellidx, maxcellidx, i;
+    int aliasidx, maxaliasidx;
     struct cell *c;
+    struct cell_alias *ca;
     int curChunk, curPage;
-    int dirSize, sizeOfCurEntry;
+    int dirSize, dotLen;
     char *newDir, *dotCell;
     struct DirHeader *dirHeader;
-    struct PageHeader *pageHeader;
-    struct DirEntry *dirEntry;
-    int doFlush = 0;
     int linkCount = 0;
     struct afs_dynSymlink *ts;
-    int newCellCount;
+    int newVersion;
 
-    /*
-     * Save afs_cellindex here, in case it changes between the
-     * two loops.
-     */
-    maxcellidx = afs_cellindex;
+    ObtainReadLock(&afs_dynrootDirLock);
+    newVersion = afs_dynrootVersion;
+    ReleaseReadLock(&afs_dynrootDirLock);
 
     /*
      * Compute the amount of space we need for the fake dir
@@ -229,39 +299,51 @@ afs_RefreshDynroot()
     curChunk = 13;
     curPage = 0;
 
-    for (cellidx = 0; cellidx < maxcellidx; cellidx++) {
-	c = afs_GetCellByIndex(cellidx, READ_LOCK, 0 /* don't refresh */);
-	if (!c) continue;
+    /* Reserve space for "." and ".." */
+    curChunk += 2;
 
-	sizeOfCurEntry = afs_dir_NameBlobs(c->cellName);
-	if (curChunk + sizeOfCurEntry > EPP) {
-	    curPage++;
-	    curChunk = 1;
-	}
-	curChunk += sizeOfCurEntry;
+    for (cellidx = 0;; cellidx++) {
+	c = afs_GetCellByIndex(cellidx, READ_LOCK);
+	if (!c)
+	    break;
+	if (c->cellNum == afs_dynrootCell)
+	    continue;
 
-	dotCell = afs_osi_Alloc(strlen(c->cellName) + 2);
+	dotLen = strlen(c->cellName) + 2;
+	dotCell = afs_osi_Alloc(dotLen);
 	strcpy(dotCell, ".");
-	strcat(dotCell, c->cellName);
-	sizeOfCurEntry = afs_dir_NameBlobs(dotCell);
-	if (curChunk + sizeOfCurEntry > EPP) {
-	    curPage++;
-	    curChunk = 1;
-	}
-	curChunk += sizeOfCurEntry;
+	afs_strcat(dotCell, c->cellName);
 
+	afs_dynroot_computeDirEnt(c->cellName, &curPage, &curChunk);
+	afs_dynroot_computeDirEnt(dotCell, &curPage, &curChunk);
+
+	afs_osi_Free(dotCell, dotLen);
 	afs_PutCell(c, READ_LOCK);
     }
+    maxcellidx = cellidx;
+
+    for (aliasidx = 0;; aliasidx++) {
+	ca = afs_GetCellAlias(aliasidx);
+	if (!ca)
+	    break;
+
+	dotLen = strlen(ca->alias) + 2;
+	dotCell = afs_osi_Alloc(dotLen);
+	strcpy(dotCell, ".");
+	afs_strcat(dotCell, ca->alias);
+
+	afs_dynroot_computeDirEnt(ca->alias, &curPage, &curChunk);
+	afs_dynroot_computeDirEnt(dotCell, &curPage, &curChunk);
+
+	afs_osi_Free(dotCell, dotLen);
+	afs_PutCellAlias(ca);
+    }
+    maxaliasidx = aliasidx;
 
     ObtainReadLock(&afs_dynSymlinkLock);
     ts = afs_dynSymlinkBase;
     while (ts) {
-	sizeOfCurEntry = afs_dir_NameBlobs(ts->name);
-	if (curChunk + sizeOfCurEntry > EPP) {
-	    curPage++;
-	    curChunk = 1;
-	}
-	curChunk += sizeOfCurEntry;
+	afs_dynroot_computeDirEnt(ts->name, &curPage, &curChunk);
 	ts = ts->next;
     }
 
@@ -273,7 +355,7 @@ afs_RefreshDynroot()
      */
     curChunk = 13;
     curPage = 0;
-    dirHeader = (struct DirHeader *) newDir;
+    dirHeader = (struct DirHeader *)newDir;
 
     dirHeader->header.pgcount = 0;
     dirHeader->header.tag = htons(1234);
@@ -281,7 +363,7 @@ afs_RefreshDynroot()
 
     dirHeader->header.freebitmap[0] = 0xff;
     dirHeader->header.freebitmap[1] = 0x1f;
-    for (i = 2; i < EPP/8; i++)
+    for (i = 2; i < EPP / 8; i++)
 	dirHeader->header.freebitmap[i] = 0;
     dirHeader->alloMap[0] = EPP - DHE - 1;
     for (i = 1; i < MAXPAGES; i++)
@@ -294,70 +376,61 @@ afs_RefreshDynroot()
     afs_dynroot_addDirEnt(dirHeader, &curPage, &curChunk, "..", 1);
     linkCount += 2;
 
-    /* Reserve space for "." and ".." */
-    curChunk += 2;
-
     for (cellidx = 0; cellidx < maxcellidx; cellidx++) {
-	c = afs_GetCellByIndex(cellidx, READ_LOCK, 0 /* don't refresh */);
-	if (!c) continue;
+	c = afs_GetCellByIndex(cellidx, READ_LOCK);
+	if (!c)
+	    continue;
+	if (c->cellNum == afs_dynrootCell)
+	    continue;
 
-	dotCell = afs_osi_Alloc(strlen(c->cellName) + 2);
+	dotLen = strlen(c->cellName) + 2;
+	dotCell = afs_osi_Alloc(dotLen);
 	strcpy(dotCell, ".");
-	strcat(dotCell, c->cellName);
-	afs_dynroot_addDirEnt(dirHeader, &curPage, &curChunk,
-			      c->cellName, VNUM_FROM_CIDX_RW(cellidx, 0));
-	afs_dynroot_addDirEnt(dirHeader, &curPage, &curChunk,
-			      dotCell, VNUM_FROM_CIDX_RW(cellidx, 1));
+	afs_strcat(dotCell, c->cellName);
+	afs_dynroot_addDirEnt(dirHeader, &curPage, &curChunk, c->cellName,
+			      VNUM_FROM_CIDX_RW(cellidx, 0));
+	afs_dynroot_addDirEnt(dirHeader, &curPage, &curChunk, dotCell,
+			      VNUM_FROM_CIDX_RW(cellidx, 1));
+	afs_osi_Free(dotCell, dotLen);
 
-	if (!(c->states & CAlias)) linkCount += 2;
+	linkCount += 2;
 	afs_PutCell(c, READ_LOCK);
     }
 
+    for (aliasidx = 0; aliasidx < maxaliasidx; aliasidx++) {
+	ca = afs_GetCellAlias(aliasidx);
+	if (!ca)
+	    continue;
+
+	dotLen = strlen(ca->alias) + 2;
+	dotCell = afs_osi_Alloc(dotLen);
+	strcpy(dotCell, ".");
+	afs_strcat(dotCell, ca->alias);
+	afs_dynroot_addDirEnt(dirHeader, &curPage, &curChunk, ca->alias,
+			      VNUM_FROM_CAIDX_RW(aliasidx, 0));
+	afs_dynroot_addDirEnt(dirHeader, &curPage, &curChunk, dotCell,
+			      VNUM_FROM_CAIDX_RW(aliasidx, 1));
+	afs_osi_Free(dotCell, dotLen);
+	afs_PutCellAlias(ca);
+    }
+
     ts = afs_dynSymlinkBase;
     while (ts) {
 	int vnum = VNUM_FROM_TYPEID(VN_TYPE_SYMLINK, ts->index);
-	afs_dynroot_addDirEnt(dirHeader, &curPage, &curChunk,
-			      ts->name, vnum);
+	afs_dynroot_addDirEnt(dirHeader, &curPage, &curChunk, ts->name, vnum);
 	ts = ts->next;
     }
 
-    newCellCount = maxcellidx + afs_dynSymlinkIndex;
     ReleaseReadLock(&afs_dynSymlinkLock);
 
     ObtainWriteLock(&afs_dynrootDirLock, 549);
-    if (afs_dynrootDir) afs_osi_Free(afs_dynrootDir, afs_dynrootDirLen);
+    if (afs_dynrootDir)
+	afs_osi_Free(afs_dynrootDir, afs_dynrootDirLen);
     afs_dynrootDir = newDir;
     afs_dynrootDirLen = dirSize;
     afs_dynrootDirLinkcnt = linkCount;
-    if (afs_dynrootCellCount != newCellCount) {
-	/*
-	 * New cells/symlinks added -- bump data version, invalidate vcache.
-	 */
-	afs_dynrootCellCount = newCellCount;
-	afs_dynrootVersion++;
-	afs_dynrootVersionHigh = osi_Time();
-	doFlush = 1;
-    }
+    afs_dynrootDirVersion = newVersion;
     ReleaseWriteLock(&afs_dynrootDirLock);
-
-    if (doFlush) {
-	afs_int32 retry;
-	struct vcache *tvc;
-	struct VenusFid tfid;
-
-	afs_GetDynrootFid(&tfid);
-	do {
-	    retry = 0;
-	    ObtainReadLock(&afs_xvcache);
-	    tvc = afs_FindVCache(&tfid, 0, 0, &retry, 0);
-	    ReleaseReadLock(&afs_xvcache);
-	} while (retry);
-	if (tvc) {
-	    tvc->states &= ~(CStatd | CUnique);
-	    osi_dnlc_purgedp(tvc);
-	    afs_PutVCache(tvc);
-	}
-    }
 }
 
 /*
@@ -365,32 +438,32 @@ afs_RefreshDynroot()
  * length thereof, and a FetchStatus.
  */
 void
-afs_GetDynroot(dynrootDir, dynrootLen, status)
-    char **dynrootDir;
-    int *dynrootLen;
-    struct AFSFetchStatus *status;
+afs_GetDynroot(char **dynrootDir, int *dynrootLen,
+	       struct AFSFetchStatus *status)
 {
     ObtainReadLock(&afs_dynrootDirLock);
-    if (!afs_dynrootDir) {
+    if (!afs_dynrootDir || afs_dynrootDirVersion != afs_dynrootVersion) {
 	ReleaseReadLock(&afs_dynrootDirLock);
-	afs_RefreshDynroot();
+	afs_RebuildDynroot();
 	ObtainReadLock(&afs_dynrootDirLock);
     }
 
-    if (dynrootDir) *dynrootDir = afs_dynrootDir;
-    if (dynrootLen) *dynrootLen = afs_dynrootDirLen;
+    if (dynrootDir)
+	*dynrootDir = afs_dynrootDir;
+    if (dynrootLen)
+	*dynrootLen = afs_dynrootDirLen;
 
     if (status) {
 	memset(status, 0, sizeof(struct AFSFetchStatus));
-	status->FileType        = Directory;
-	status->LinkCount       = afs_dynrootDirLinkcnt;
-	status->Length          = afs_dynrootDirLen;
-	status->DataVersion     = afs_dynrootVersion;
-	status->CallerAccess    = PRSFS_LOOKUP | PRSFS_READ;
+	status->FileType = Directory;
+	status->LinkCount = afs_dynrootDirLinkcnt;
+	status->Length = afs_dynrootDirLen;
+	status->DataVersion = afs_dynrootVersion;
+	status->CallerAccess = PRSFS_LOOKUP | PRSFS_READ;
 	status->AnonymousAccess = PRSFS_LOOKUP | PRSFS_READ;
-	status->UnixModeBits    = 0755;
-	status->ParentVnode     = 1;
-	status->ParentUnique    = 1;
+	status->UnixModeBits = 0755;
+	status->ParentVnode = 1;
+	status->ParentUnique = 1;
 	status->dataVersionHigh = afs_dynrootVersionHigh;
     }
 }
@@ -399,7 +472,7 @@ afs_GetDynroot(dynrootDir, dynrootLen, status)
  * Puts back the dynroot read lock.
  */
 void
-afs_PutDynroot()
+afs_PutDynroot(void)
 {
     ReleaseReadLock(&afs_dynrootDirLock);
 }
@@ -410,11 +483,10 @@ afs_PutDynroot()
  * FetchStatus will be filled in.
  */
 int
-afs_DynrootNewVnode(avc, status)
-    struct vcache *avc;
-    struct AFSFetchStatus *status;
+afs_DynrootNewVnode(struct vcache *avc, struct AFSFetchStatus *status)
 {
-    if (!afs_dynrootEnable) return 0;
+    if (!afs_dynrootEnable)
+	return 0;
 
     if (afs_IsDynroot(avc)) {
 	afs_GetDynroot(0, 0, status);
@@ -425,21 +497,22 @@ afs_DynrootNewVnode(avc, status)
     /*
      * Check if this is an entry under /afs, e.g. /afs/cellname.
      */
-    if (avc->fid.Cell       == AFS_DYNROOT_CELL &&
-	avc->fid.Fid.Volume == AFS_DYNROOT_VOLUME) {
+    if (avc->fid.Cell == afs_dynrootCell
+	&& avc->fid.Fid.Volume == AFS_DYNROOT_VOLUME) {
 
 	struct cell *c;
+	struct cell_alias *ca;
 	int namelen, linklen, cellidx, rw;
 
 	memset(status, 0, sizeof(struct AFSFetchStatus));
 
-	status->FileType        = SymbolicLink;
-	status->LinkCount       = 1;
-	status->DataVersion     = 1;
-	status->CallerAccess    = PRSFS_LOOKUP | PRSFS_READ;
+	status->FileType = SymbolicLink;
+	status->LinkCount = 1;
+	status->DataVersion = 1;
+	status->CallerAccess = PRSFS_LOOKUP | PRSFS_READ;
 	status->AnonymousAccess = PRSFS_LOOKUP | PRSFS_READ;
-	status->ParentVnode     = 1;
-	status->ParentUnique    = 1;
+	status->ParentVnode = 1;
+	status->ParentUnique = 1;
 
 	if (VNUM_TO_VNTYPE(avc->fid.Fid.Vnode) == VN_TYPE_SYMLINK) {
 	    struct afs_dynSymlink *ts;
@@ -448,7 +521,8 @@ afs_DynrootNewVnode(avc, status)
 	    ObtainReadLock(&afs_dynSymlinkLock);
 	    ts = afs_dynSymlinkBase;
 	    while (ts) {
-		if (ts->index == index) break;
+		if (ts->index == index)
+		    break;
 		ts = ts->next;
 	    }
 
@@ -457,7 +531,7 @@ afs_DynrootNewVnode(avc, status)
 		avc->linkData = afs_osi_Alloc(linklen + 1);
 		strcpy(avc->linkData, ts->target);
 
-		status->Length       = linklen;
+		status->Length = linklen;
 		status->UnixModeBits = 0755;
 	    }
 	    ReleaseReadLock(&afs_dynSymlinkLock);
@@ -465,7 +539,8 @@ afs_DynrootNewVnode(avc, status)
 	    return ts ? 1 : 0;
 	}
 
-	if (VNUM_TO_VNTYPE(avc->fid.Fid.Vnode) != VN_TYPE_CELL) {
+	if (VNUM_TO_VNTYPE(avc->fid.Fid.Vnode) != VN_TYPE_CELL
+	    && VNUM_TO_VNTYPE(avc->fid.Fid.Vnode) != VN_TYPE_ALIAS) {
 	    afs_warn("dynroot vnode inconsistency, unknown VNTYPE %d\n",
 		     VNUM_TO_VNTYPE(avc->fid.Fid.Vnode));
 	    return 0;
@@ -474,36 +549,44 @@ afs_DynrootNewVnode(avc, status)
 	cellidx = VNUM_TO_CIDX(avc->fid.Fid.Vnode);
 	rw = VNUM_TO_RW(avc->fid.Fid.Vnode);
 
-	c = afs_GetCellByIndex(cellidx, READ_LOCK, 1 /* refresh */);
-	if (!c) {
-	    afs_warn("dynroot vnode inconsistency, can't find cell %d\n",
-		     cellidx);
-	    return 0;
-	}
+	if (VNUM_TO_VNTYPE(avc->fid.Fid.Vnode) == VN_TYPE_ALIAS) {
+	    char *realName;
+
+	    ca = afs_GetCellAlias(cellidx);
+	    if (!ca) {
+		afs_warn("dynroot vnode inconsistency, can't find alias %d\n",
+			 cellidx);
+		return 0;
+	    }
 
-	if (c->states & CAlias) {
 	    /*
 	     * linkData needs to contain the name of the cell
 	     * we're aliasing for.
 	     */
-	    char *realName = c->realName;
-
+	    realName = ca->cell;
 	    if (!realName) {
 		afs_warn("dynroot: alias %s missing real cell name\n",
-			 c->cellName);
+			 ca->alias);
+		avc->linkData = afs_strdup("unknown");
 		linklen = 7;
-		avc->linkData = afs_osi_Alloc(linklen + 1);
-		strcpy(avc->linkData, "unknown");
 	    } else {
 		int namelen = strlen(realName);
 		linklen = rw + namelen;
 		avc->linkData = afs_osi_Alloc(linklen + 1);
 		strcpy(avc->linkData, rw ? "." : "");
-		strcat(avc->linkData, realName);
+		afs_strcat(avc->linkData, realName);
 	    }
 
 	    status->UnixModeBits = 0755;
+	    afs_PutCellAlias(ca);
 	} else {
+	    c = afs_GetCellByIndex(cellidx, READ_LOCK);
+	    if (!c) {
+		afs_warn("dynroot vnode inconsistency, can't find cell %d\n",
+			 cellidx);
+		return 0;
+	    }
+
 	    /*
 	     * linkData needs to contain "#cell:root.cell" or "%cell:root.cell"
 	     */
@@ -511,14 +594,14 @@ afs_DynrootNewVnode(avc, status)
 	    linklen = 1 + namelen + 10;
 	    avc->linkData = afs_osi_Alloc(linklen + 1);
 	    strcpy(avc->linkData, rw ? "%" : "#");
-	    strcat(avc->linkData, c->cellName);
-	    strcat(avc->linkData, ":root.cell");
+	    afs_strcat(avc->linkData, c->cellName);
+	    afs_strcat(avc->linkData, ":root.cell");
 
 	    status->UnixModeBits = 0644;
+	    afs_PutCell(c, READ_LOCK);
 	}
 
 	status->Length = linklen;
-	afs_PutCell(c, READ_LOCK);
 	return 1;
     }
 
@@ -529,18 +612,17 @@ afs_DynrootNewVnode(avc, status)
  * Enable or disable dynroot.  Returns 0 if successful.
  */
 int
-afs_SetDynrootEnable(enable)
-    int enable;
+afs_SetDynrootEnable(int enable)
 {
     afs_dynrootEnable = enable;
-    return 0;
+    return afs_dynrootCellInit();
 }
 
 /*
  * Check if dynroot support is enabled.
  */
 int
-afs_GetDynrootEnable()
+afs_GetDynrootEnable(void)
 {
     return afs_dynrootEnable;
 }
@@ -549,14 +631,10 @@ afs_GetDynrootEnable()
  * Remove a temporary symlink entry from /afs.
  */
 int
-afs_DynrootVOPRemove(avc, acred, aname)
-    struct vcache *avc;
-    struct AFS_UCRED *acred;
-    char *aname;
+afs_DynrootVOPRemove(struct vcache *avc, struct AFS_UCRED *acred, char *aname)
 {
     struct afs_dynSymlink **tpps;
     struct afs_dynSymlink *tps;
-    struct cell *c;
     int found = 0;
 
     if (acred->cr_uid)
@@ -579,42 +657,29 @@ afs_DynrootVOPRemove(avc, acred, aname)
     }
     ReleaseWriteLock(&afs_dynSymlinkLock);
     if (found) {
-	afs_RefreshDynroot();
+	afs_DynrootInvalidate();
 	return 0;
     }
 
-    /* Check if this is an actual cell? */
-    c = afs_GetCellByName2(aname, READ_LOCK, 0 /* no AFSDB */);
-    if (c) {
-	afs_PutCell(c, READ_LOCK);
+    if (afs_CellOrAliasExists(aname))
 	return EROFS;
-    } else {
+    else
 	return ENOENT;
-    }
 }
 
 /*
  * Create a temporary symlink entry in /afs.
  */
 int
-afs_DynrootVOPSymlink(avc, acred, aname, atargetName)
-    struct vcache *avc;
-    struct AFS_UCRED *acred;
-    char *aname;
-    char *atargetName;
+afs_DynrootVOPSymlink(struct vcache *avc, struct AFS_UCRED *acred,
+		      char *aname, char *atargetName)
 {
     struct afs_dynSymlink *tps;
-    struct cell *c;
 
     if (acred->cr_uid)
 	return EPERM;
-
-    /* Check if it's already a cell */
-    c = afs_GetCellByName2(aname, READ_LOCK, 0 /* no AFSDB */);
-    if (c) {
-	afs_PutCell(c, READ_LOCK);
+    if (afs_CellOrAliasExists(aname))
 	return EEXIST;
-    }
 
     /* Check if it's already a symlink */
     ObtainWriteLock(&afs_dynSymlinkLock, 91);
@@ -638,6 +703,6 @@ afs_DynrootVOPSymlink(avc, acred, aname, atargetName)
     afs_dynSymlinkBase = tps;
     ReleaseWriteLock(&afs_dynSymlinkLock);
 
-    afs_RefreshDynroot();
+    afs_DynrootInvalidate();
     return 0;
 }
