@@ -17,7 +17,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/lwp/lwp.c,v 1.26 2004/07/08 05:11:34 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/lwp/lwp.c,v 1.27.2.3 2005/01/31 04:20:00 shadow Exp $");
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -73,6 +73,10 @@ extern char PRE_Block;		/* from preempt.c */
 #define MINFRAME 128
 #define STACK_ALIGN 8
 #else
+#ifdef __s390x__
+#define MINFRAME    160
+#define STACK_ALIGN 8
+#else
 #ifdef __s390__
 #define MINFRAME    96
 #define STACK_ALIGN 8
@@ -80,6 +84,7 @@ extern char PRE_Block;		/* from preempt.c */
 #define STACK_ALIGN 4
 #endif
 #endif
+#endif
 
 /* Debugging macro */
 #ifdef DEBUG
@@ -237,7 +242,7 @@ LWP_QWait(void)
 {
     register PROCESS tp;
     (tp = lwp_cpptr)->status = QWAITING;
-    move(tp, &runnable[tp->priority], qwaiting);
+    move(tp, &runnable[tp->priority], &qwaiting);
     Set_LWP_RC();
     return LWP_SUCCESS;
 }
@@ -248,7 +253,7 @@ LWP_QSignal(pid)
 {
     if (pid->status == QWAITING) {
 	pid->status = READY;
-	move(pid, qwaiting, &runnable[pid->priority]);
+	move(pid, &qwaiting, &runnable[pid->priority]);
 	return LWP_SUCCESS;
     } else
 	return LWP_ENOWAIT;
