@@ -83,7 +83,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/viced/callback.c,v 1.54 2003/11/23 04:53:41 jaltman Exp $");
+    ("$Header: /cvs/openafs/src/viced/callback.c,v 1.55.2.5 2005/01/31 04:10:43 shadow Exp $");
 
 #include <stdio.h>
 #include <stdlib.h>		/* for malloc() */
@@ -572,6 +572,7 @@ XCallBackBulk_r(struct host * ahost, struct AFSFid * fids, afs_int32 nfids)
     struct AFSCBs tc;
     int code;
     int j;
+    struct rx_connection *cb_conn = NULL;
 
 #ifdef	ADAPT_MTU
     rx_SetConnDeadTime(ahost->callback_rxcon, 4);
@@ -594,8 +595,12 @@ XCallBackBulk_r(struct host * ahost, struct AFSFid * fids, afs_int32 nfids)
 	tc.AFSCBs_len = i;
 	tc.AFSCBs_val = tcbs;
 
+	cb_conn = ahost->callback_rxcon;
+	rx_GetConnection(cb_conn);
 	H_UNLOCK;
-	code |= RXAFSCB_CallBack(ahost->callback_rxcon, &tf, &tc);
+	code |= RXAFSCB_CallBack(cb_conn, &tf, &tc);
+	rx_PutConnection(cb_conn);
+	cb_conn = NULL;
 	H_LOCK;
     }
 
@@ -776,6 +781,7 @@ MultiBreakCallBack_r(struct cbstruct cba[], int ncbas,
 	if (!thishost || (thishost->hostFlags & HOSTDELETED)) {
 	    continue;
 	}
+	rx_GetConnection(thishost->callback_rxcon);
 	conns[j++] = thishost->callback_rxcon;
 
 #ifdef	ADAPT_MTU
@@ -832,14 +838,14 @@ MultiBreakCallBack_r(struct cbstruct cba[], int ncbas,
 			}
 
 			H_LOCK;
-			h_Lock_r(hp);
+			h_Lock_r(hp); 
 			hp->hostFlags |= VENUSDOWN;
 		/**
 		  * We always go into AddCallBack1_r with the host locked
 		  */
 			AddCallBack1_r(hp, afidp->AFSCBFids_val, itot(idx),
 				       CB_DELAYED, 1);
-			h_Unlock_r(hp);
+			h_Unlock_r(hp); 
 			H_UNLOCK;
 		    }
 		}
@@ -853,10 +859,21 @@ MultiBreakCallBack_r(struct cbstruct cba[], int ncbas,
     for (i = 0; i < ncbas; i++) {
 	struct host *hp;
 	hp = cba[i].hp;
-	if (hp && xhost != hp)
+	if (hp && xhost != hp) {
 	    h_Release_r(hp);
+	}
     }
 
+    /* H_UNLOCK around this so h_FreeConnection does not deadlock.
+       h_FreeConnection should *never* be called on a callback connection,
+       but on 10/27/04 a deadlock occurred where it was, when we know why,
+       this should be reverted. -- shadow */
+    H_UNLOCK;
+    for (i = 0; i < j; i++) {
+	rx_PutConnection(conns[i]);
+    }
+    H_LOCK;
+
     return;
 }
 
@@ -1073,21 +1090,24 @@ BreakDelayedCallBacks_r(struct host *host)
     struct CallBack *cb;
     int code;
     char hoststr[16];
+    struct rx_connection *cb_conn;
 
     cbstuff.nbreakers++;
     if (!(host->hostFlags & RESETDONE) && !(host->hostFlags & HOSTDELETED)) {
 	host->hostFlags &= ~ALTADDR;	/* alterrnate addresses are invalid */
+	cb_conn = host->callback_rxcon;
+	rx_GetConnection(cb_conn);
 	if (host->interface) {
 	    H_UNLOCK;
 	    code =
-		RXAFSCB_InitCallBackState3(host->callback_rxcon,
-					   &FS_HostUUID);
-	    H_LOCK;
+		RXAFSCB_InitCallBackState3(cb_conn, &FS_HostUUID);
 	} else {
 	    H_UNLOCK;
-	    code = RXAFSCB_InitCallBackState(host->callback_rxcon);
-	    H_LOCK;
+	    code = RXAFSCB_InitCallBackState(cb_conn);
 	}
+	rx_PutConnection(cb_conn);
+	cb_conn = NULL;
+	H_LOCK;
 	host->hostFlags |= ALTADDR;	/* alternate addresses are valid */
 	if (code) {
 	    if (ShowProblems) {
@@ -1489,6 +1509,7 @@ CleanupTimedOutCallBacks(void)
     H_LOCK;
     CleanupTimedOutCallBacks_r();
     H_UNLOCK;
+    return 0;
 }
 
 int
@@ -1531,21 +1552,21 @@ CleanupTimedOutCallBacks_r(void)
 }
 
 static struct host *lih_host;
-static int lih_host_held = 0;
+static int lih_host_held;
 
 static int
 lih_r(register struct host *host, register int held,
       register struct host *hostp)
 {
-    lih_host_held = 0;
     if (host->cblist
 	&& ((hostp && host != hostp) || (!held && !h_OtherHolds_r(host)))
 	&& (!lih_host || host->ActiveCall < lih_host->ActiveCall)) {
+	if (lih_host != NULL && lih_host_held) {
+	    h_Release_r(lih_host);
+	}
 	lih_host = host;
-    }
-    if (!held) {
+	lih_host_held = !held;
 	held = 1;
-	lih_host_held = 1;
     }
     return held;
 }
@@ -1573,13 +1594,15 @@ GetSomeSpace_r(struct host *hostp, int locked)
 	h_Enumerate_r(lih_r, hp2, (char *)hp1);
 	hp = lih_host;
 	if (hp) {
+	    /* set in lih_r! private copy before giving up H_LOCK */
+	    int lih_host_held2=lih_host_held;   
 	    cbstuff.GSS4++;
 	    if (!ClearHostCallbacks_r(hp, 0 /* not locked or held */ )) {
-		if (lih_host_held) 
+		if (lih_host_held2)
 		    h_Release_r(hp);
 		return 0;
 	    }
-	    if (lih_host_held) 
+	    if (lih_host_held2)
 		h_Release_r(hp);
 	    hp2 = hp->next;
 	} else {
@@ -1622,6 +1645,7 @@ ClearHostCallbacks_r(struct host *hp, int locked)
     int code;
     int held = 0;
     char hoststr[16];
+    struct rx_connection *cb_conn = NULL;
 
     ViceLog(5,
 	    ("GSS: Delete longest inactive host %s\n",
@@ -1657,16 +1681,19 @@ ClearHostCallbacks_r(struct host *hp, int locked)
     } else {
 	/* host is up, try a call */
 	hp->hostFlags &= ~ALTADDR;	/* alternate addresses are invalid */
+	cb_conn = hp->callback_rxcon;
+	rx_GetConnection(hp->callback_rxcon);
 	if (hp->interface) {
 	    H_UNLOCK;
 	    code =
-		RXAFSCB_InitCallBackState3(hp->callback_rxcon, &FS_HostUUID);
-	    H_LOCK;
+		RXAFSCB_InitCallBackState3(cb_conn, &FS_HostUUID);
 	} else {
 	    H_UNLOCK;
-	    code = RXAFSCB_InitCallBackState(hp->callback_rxcon);
-	    H_LOCK;
+	    code = RXAFSCB_InitCallBackState(cb_conn);
 	}
+	rx_PutConnection(cb_conn);
+	cb_conn = NULL;
+	H_LOCK;
 	hp->hostFlags |= ALTADDR;	/* alternate addresses are valid */
 	if (code) {
 	    /* failed, mark host down and need reset */
